
/*
 *     MPEG Video Server 
 *
 *     Date Started: 03/01/95
 *
 *     Author - Mahesh Subramanyan
 *            - Miguel del Rosario
 *            - Marek Podgorny
 *              NPAC, Syracuse University
 *     
 *     04/25/95 - Modified to compile on nCube. The overall logic
 *     remains the same. This is not a concurrent server. One
 *     environment variable, NSOCKET needs to be set to 0x8003 and use
 *     the switch -NBSD on the compile command. 
 *
 *     05/15/95 - The code works on nCube. Able to transmit MPEG
 *     systems stream to a SGI Client. Resolved problems related to
 *     nCube being little endian and SGI being big endian.
 *
*/

#include <stdio.h>
#include <malloc.h>
#include <netdb.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>

#define MAX_BUF             200000
#define CONFIG_SIZE         5

char *pname;

main(int argc, char *argv[])
{
  
  struct sockaddr_in  serv_addr, client_addr;
  char   *message, moviename[255];
  int    sockfd, newsockfd, clilen, childpid, len, fd;
  int    sndsize, rcvsize;
  int    optval = 1;
  int    config[CONFIG_SIZE];
  int    snd, rcv, optlen, sendbuff;
  int i, temp;
  
  pname = argv[0];
  
  if ( argc != 2 ) {
    fprintf(stderr, "Usage: tcp_server port\n");
    exit(1);
  }
  
  if ((message = (char *)malloc(MAX_BUF)) == NULL) {
    perror("server: error in malloc");
    exit(1);
  }

  if ( ( sockfd = socket(AF_INET, SOCK_STREAM, 0) ) < 0 ) {
    fprintf(stderr, "server: can't open stream socket.\n");
    exit(1);
  }
/*
  setsockopt(sockfd, IPPROTO_TCP, TCP_NODELAY, (char *)&optval, sizeof(optval));
*/ 
  bzero((char *) &serv_addr, sizeof(serv_addr)); 
  serv_addr.sin_family      = AF_INET;          
  serv_addr.sin_port        = htons(atoi(argv[1]));   
  serv_addr.sin_addr.s_addr = htonl(INADDR_ANY); 
  
  if ( bind(sockfd,(struct sockaddr *) &serv_addr,sizeof(serv_addr)) < 0 ) {
    fprintf(stderr, "server: can't bind local address.");
    exit(1);
  }
  
  if (listen(sockfd, 0) < 0) {
    fprintf(stderr, "server: can't listen\n");
    exit(1);
  }
  
  clilen = sizeof(client_addr);
#ifdef DEBUG
  fprintf(stderr, "before accept\n");
#endif
  newsockfd = accept(sockfd, (struct sockaddr *) &client_addr, &clilen);
  if ( newsockfd < 0 ) {
    perror("");
    exit(1);
  }
/*
  setsockopt(newsockfd, IPPROTO_TCP, TCP_NODELAY, (char *) &optval,
      sizeof(optval));
  snd = rcv = 8192;
  setsockopt(newsockfd, SOL_SOCKET, SO_SNDBUF, (char *) &snd,
      sizeof(snd));
  setsockopt(newsockfd, SOL_SOCKET, SO_RCVBUF, (char *) &rcv,
	  sizeof(rcv));
*/
#ifdef DEBUG
  optlen = sizeof(sendbuff);
  if (getsockopt(sockfd, SOL_SOCKET, SO_SNDBUF, (char *) &sendbuff, &optlen) < 0) 
    perror("SO_SNDBUF getsockopt error");
  fprintf(stderr, "send buffer size = %d\n", sendbuff);
  optlen = sizeof(sendbuff);
  if (getsockopt(sockfd, SOL_SOCKET, SO_RCVBUF, (char *) &sendbuff, &optlen) < 0) 
    perror("SO_SNDBUF getsockopt error");
  fprintf(stderr, "receive buffer size = %d\n", sendbuff);
#endif
  if ( readn(newsockfd, config, sizeof(config)) < 0 ) {
    fprintf(stderr, "server: can't read moviename from the socket.\n");
    close(newsockfd);
    exit(1);
  }
  config[0] = ntohl(config[0]);
#ifdef DEBUG
  fprintf(stderr, "%d\n", config[0]);
#endif
  if ( readn(newsockfd, moviename, config[0]) < 0 ) {
    fprintf(stderr, "server: can't read %s from the socket.\n", moviename);
    close(newsockfd);
    exit(1);
  }

  if ((fd = open(moviename, O_RDONLY, 0)) < 0) {
    fprintf(stderr, "error opening movie file\n");
    exit(1);
  }
#ifdef DEBUG
  fprintf(stderr, "Playing movie - %s\n", moviename);
#endif

  while ( 1 ) {
    if ( readn(newsockfd, config, sizeof(config)) < 0 ) {
      fprintf(stderr, "server: can't read size from the socket.\n");
      close(newsockfd);
      exit(1);
    }
    sndsize = ntohl(config[0]); 
#ifdef DEBUG
    fprintf(stderr, "before read - sndsize = %d\n", sndsize);
#endif
    if ( ( len = read(fd, message, sndsize) ) < 0 ) {
      fprintf(stderr, "server: can't read from %s.\n", moviename);
      close(newsockfd);
      exit(1);
    }
#ifdef DEBUG
    fprintf(stderr, "after read - len = %d\n", len);
#endif
    if ((len == 0) | (len < sndsize)) {
      close(newsockfd);
      fprintf(stderr, "End of %s\n", moviename);
      exit(1);
    }
    if ( (temp = writen(newsockfd, message, len)) < 0 ) {
      fprintf(stderr, "server: can't write into the socket.");
      close(newsockfd);
      exit(1);
    }
#ifdef DEBUG
    fprintf(stderr, "after write - temp = %d\n", temp);
#endif
  }
  close(newsockfd);  
}

